import pytest
import responses
import requests

from django.urls import reverse
from django.test.client import RequestFactory, Client
from wspay.conf import settings, resolve

from wspay.forms import UnprocessedPaymentForm, WSPaySignedForm
from wspay.models import WSPayRequest
from wspay.services import generate_wspay_form_data, generate_signature, get_endpoint

from wspay.tests.utils import TRANSACTION_REPORT


def test_incoming_data_form():
    """Test the form that receives cart and user details."""
    form = UnprocessedPaymentForm()
    assert form.is_valid() is False
    form = UnprocessedPaymentForm({'user_id': 1, 'cart_id': 1, 'price': 1})
    assert form.is_valid()


@pytest.mark.django_db
def test_wspay_encode():
    """Test the processing function which prepares the data for WSPay."""
    shop_id = resolve(settings.WS_PAY_SHOP_ID)
    secret_key = resolve(settings.WS_PAY_SECRET_KEY)
    assert shop_id == 'MojShop'
    assert secret_key == 'MojSecret'

    return_data = {
        'ShopID': shop_id,
        'Version': resolve(settings.WS_PAY_VERSION),
        'TotalAmount': '10,00',
        'ReturnURL': (
            'http://testserver' + reverse('wspay:process-response', kwargs={'status': 'success'})
        ),
        'CancelURL': (
            'http://testserver' + reverse('wspay:process-response', kwargs={'status': 'cancel'})
        ),
        'ReturnErrorURL': (
            'http://testserver' + reverse('wspay:process-response', kwargs={'status': 'error'})
        ),
        'ReturnMethod': 'POST',
    }

    incoming_form = UnprocessedPaymentForm({'cart_id': 1, 'price': 10})
    if (incoming_form.is_valid()):
        form_data = generate_wspay_form_data(
            incoming_form.cleaned_data.copy(), RequestFactory().get('/')
        )

    req = WSPayRequest.objects.get()
    return_data['ShoppingCartID'] = str(req.request_uuid)
    return_data['Signature'] = generate_signature([
        shop_id,
        secret_key,
        str(req.request_uuid),
        secret_key,
        '1000',
        secret_key,
    ])

    assert return_data == form_data


@pytest.mark.django_db
def test_wspay_form():
    """Test the form that is used to make a WSPay POST request."""
    form = WSPaySignedForm()
    assert form.is_valid() is False

    incoming_form = UnprocessedPaymentForm({'user_id': 1, 'cart_id': 1, 'price': 1})
    if (incoming_form.is_valid()):
        form_data = generate_wspay_form_data(
            incoming_form.cleaned_data.copy(),
            RequestFactory().get('/')
        )

    form = WSPaySignedForm(form_data)
    assert form.is_valid()
    form = form.cleaned_data

    responses.add(responses.POST, 'https://formtest.wspay.biz/authorization.aspx', status=200)
    response = requests.post('https://formtest.wspay.biz/authorization.aspx', form)
    assert response.status_code == 200


@pytest.mark.django_db
def test_transaction_update(settings):
    """Test wspay transaction update callback."""
    request = WSPayRequest.objects.create(cart_id=1)
    settings.WS_PAY_SHOP_ID = 'MYSHOP'
    settings.WS_PAY_SECRET_KEY = '3DfEO2B5Jjm4VC1Q3vEh'
    TRANSACTION_REPORT['ShoppingCartID'] = str(request.request_uuid)
    r = Client().post(
        reverse('wspay:transaction-report'),
        TRANSACTION_REPORT,
    )
    assert r.status_code == 200
    assert r.content == b'OK'
    assert request.transactions.count() == 1


def test_conf_resolver():
    """Test conf resolver when settings are callables or dotted path to a callable."""
    assert resolve(settings.WS_PAY_SHOP_ID) == 'MojShop'
    assert resolve(settings.WS_PAY_SECRET_KEY) == 'MojSecret'


def test_get_endpoint(settings):
    """Test get_endpoint setting."""
    assert settings.WS_PAY_DEVELOPMENT is True
    assert get_endpoint() == 'https://formtest.wspay.biz/authorization.aspx'

    settings.WS_PAY_DEVELOPMENT = False
    assert get_endpoint() == 'https://form.wspay.biz/authorization.aspx'



TRANSACTION_REPORT = {
    'WsPayOrderId': '96a5f58f-764c-4640-90ea-591280893bff',
    'UniqueTransactionNumber': 46370,
    'Signature': ''.join([
        '719cbb3ea1edb3cde1bf839c39be2be9fc43f96d317f862007b341862e623870110c1e2b27d556394',
        'ae76543d18a10120f6f9ea9753af4086aca036ef75010c1'
    ]),
    'STAN': '38967',
    'ApprovalCode': '961792',
    'ShopID': 'MYSHOP',
    'ShoppingCartID': '1c731772-e407-45f8-b576-7e20b0e8642d',
    'Amount': 78,
    'CurrencyCode': 191,
    'ActionSuccess': '1',
    'Success': '1',
    'Authorized': '1',
    'Completed': '0',
    'Voided': '0',
    'Refunded': '0',
    'PaymentPlan': '0000',
    'Partner': 'Pbz',
    'OnSite': '1',
    'CreditCardName': 'AMEX',
    'CreditCardNumber': '377500*****1007',
    'ECI': '',
    'CustomerFirstName': 'John',
    'CustomerLastName': 'Doe',
    'CustomerAddress': 'Street address 10',
    'CustomerCity': 'City',
    'CustomerCountry': 'HR',
    'CustomerPhone': '0911111111111111',
    'CustomerZIP': '51000',
    'CustomerEmail': 'john@doe.com',
    'TransactionDateTime': '20200423102457',
    'IsLessThen30DaysFromTransaction': True,
    'CanBeCompleted': True,
    'CanBeVoided': True,
    'CanBeRefunded': False,
    'Token': '09bf3d95-7a57-43f3-873a-216467a39925',
    'TokenNumber': '1007',
    'ExpirationDate': '2006'
}


def secret_key():
    """Resolve secret key setting."""
    return 'MojSecret'
